import rkstiff.grids 
import numpy as np

def test_hankel():
    HT = rkstiff.grids.HankelTransform(nr=50,rmax=2.0)
    a = 4

    f1 = np.exp(-a**2*HT.r**2)
    fsp1_ex = np.exp(-HT.kr**2/(4*a**2))/(2*a**2)
    fsp1 = HT.ht(f1)
    error1 = np.linalg.norm(fsp1-fsp1_ex)/np.linalg.norm(fsp1_ex)
    assert error1 < 1e-10

    HT.rmax = 4
    sigma = 2
    w = 0.5
    f2 = np.exp(-sigma*HT.r**2)*np.sin(w*HT.r**2)
    omega = 1./(4*(sigma**2+w**2))
    fsp2_ex = -2*omega*np.exp(-sigma*omega*HT.kr**2)*(-w*np.cos(w*omega*HT.kr**2)+sigma*np.sin(w*omega*HT.kr**2))
    fsp2 = HT.ht(f2)
    error2 = np.linalg.norm(fsp2-fsp2_ex)/np.linalg.norm(fsp2_ex)
    assert error2 < 1e-10


    HT.nr = 25
    HT.rmax = 3
    f3 = np.exp(-a*HT.r)
    fsp3_ex = a*np.power(a**2+HT.kr**2,-3./2)
    fsp3 = HT.ht(f3)
    error3 = np.linalg.norm(fsp3-fsp3_ex)/np.linalg.norm(fsp3_ex)
    assert error3 < 1e-2










from rkstiff.grids import construct_x_kx_rfft, construct_x_kx_fft
from rkstiff.grids import construct_x_Dx_cheb
from rkstiff.derivatives import dx_rfft, dx_fft
import numpy as np

def test_periodic_dx_rfft():
    N = 100
    a, b = 0, 2*np.pi
    x,kx = construct_x_kx_rfft(N,a,b)
    u = np.sin(x)
    ux_exact = np.cos(x)
    ux_approx = dx_rfft(kx,u)
    assert np.allclose(ux_exact,ux_approx)

def test_zeroboundaries_dx_rfft():
    N = 400
    a, b = -30., 30.
    x,kx = construct_x_kx_rfft(N,a,b)
    u = 1./np.cosh(x)
    ux_exact = -np.tanh(x)/np.cosh(x)
    ux_approx = dx_rfft(kx,u)
    assert np.allclose(ux_exact,ux_approx)

def test_gauss_dx_rfft():
    N = 128
    a,b = -10,10
    x,kx = construct_x_kx_rfft(N,a,b)
    u = np.exp(-x**2)
    ux_exact = -2*x*np.exp(-x**2)
    ux_approx = dx_rfft(kx,u)
    assert np.allclose(ux_exact,ux_approx)


def test_manydx_rfft():
    N = 128 
    a, b = 0, 2*np.pi
    x,kx = construct_x_kx_rfft(N,a,b)
    u = np.sin(x)
    ux_exact = np.sin(x)

    ux_approx = u.copy()
    for _ in range(4):
        ux_approx = dx_rfft(kx,ux_approx)
    rel_err = np.linalg.norm(ux_exact-ux_approx)/np.linalg.norm(ux_exact) 
    assert rel_err < 1e-6

    ux_approx = u.copy()
    ux_approx = dx_rfft(kx,ux_approx,8)
    rel_err = np.linalg.norm(ux_exact-ux_approx)/np.linalg.norm(ux_exact) 
    assert rel_err < 0.1




def test_manydx_fft():
    N = 128 
    a, b = 0, 2*np.pi
    x,kx = construct_x_kx_fft(N,a,b)
    u = np.sin(x)
    ux_exact = np.sin(x)

    ux_approx = u.copy()
    for _ in range(4):
        ux_approx = dx_fft(kx,ux_approx)
    rel_err = np.linalg.norm(ux_exact-ux_approx)/np.linalg.norm(ux_exact) 
    assert rel_err < 1e-6

    ux_approx = u.copy()
    ux_approx = dx_fft(kx,ux_approx,8)
    rel_err = np.linalg.norm(ux_exact-ux_approx)/np.linalg.norm(ux_exact) 
    assert rel_err < 0.1 

def test_periodic_dx_fft():
    N = 100
    a, b = 0, 2*np.pi
    x,kx = construct_x_kx_fft(N,a,b)
    u = np.sin(x)
    ux_exact = np.cos(x)
    ux_approx = dx_fft(kx,u)
    assert np.allclose(ux_exact,ux_approx)

def test_zeroboundaries_dx_fft():
    N = 400
    a, b = -30., 30.
    x,kx = construct_x_kx_fft(N,a,b)
    u = 1./np.cosh(x)
    ux_exact = -np.tanh(x)/np.cosh(x)
    ux_approx = dx_fft(kx,u)
    assert np.allclose(ux_exact,ux_approx)

def test_gauss_dx_fft():
    N = 128
    a,b = -10,10
    x,kx = construct_x_kx_fft(N,a,b)
    u = np.exp(-x**2)
    ux_exact = -2*x*np.exp(-x**2)
    ux_approx = dx_fft(kx,u)
    assert np.allclose(ux_exact,ux_approx)


def test_exp_trig_x_Dx_cheb():
    # standard interval [-1,1]
    N = 20; a = -1; b = 1
    x,Dx = construct_x_Dx_cheb(N,-1,1)
    u = np.exp(x)*np.sin(5*x)
    Du_exact = np.exp(x)*(np.sin(5*x)+5*np.cos(5*x))
    Du_approx = Dx.dot(u) 
    error = Du_exact - Du_approx
    assert np.linalg.norm(error)/np.linalg.norm(Du_exact) < 1e-8

    # non-standard interval [-3,3]
    N = 30; a = -3; b = 3
    x,Dx = construct_x_Dx_cheb(N,a,b)
    u = np.exp(x)*np.sin(5*x)
    Du_exact = np.exp(x)*(np.sin(5*x)+5*np.cos(5*x))
    Du_approx = Dx.dot(u) 
    error = Du_exact - Du_approx
    assert np.linalg.norm(error)/np.linalg.norm(Du_exact) < 1e-7


